@@ -1,7 +1,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
+# Most of this work is copyright (C) 2013-2021 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -17,7 +17,7 @@ from inspect import signature
 
 import pytest
 
-from hypothesis import Verbosity, core, settings
+from hypothesis import HealthCheck, Verbosity, core, settings
 from hypothesis._settings import note_deprecation
 from hypothesis.errors import InvalidArgument
 from hypothesis.internal.detection import is_hypothesis_test
@@ -86,17 +86,15 @@ else:
         )
 
     def pytest_report_header(config):
+        if config.option.verbose < 1 and settings.default.verbosity < Verbosity.verbose:
+            return None
         profile = config.getoption(LOAD_PROFILE_OPTION)
         if not profile:
             profile = settings._current_profile
         settings_str = settings.get_profile(profile).show_changed()
         if settings_str != "":
             settings_str = " -> %s" % (settings_str)
-        if (
-            config.option.verbose >= 1
-            or settings.default.verbosity >= Verbosity.verbose
-        ):
-            return "hypothesis profile %r%s" % (profile, settings_str)
+        return "hypothesis profile %r%s" % (profile, settings_str)
 
     def pytest_configure(config):
         core.running_under_pytest = True
@@ -148,30 +146,46 @@ else:
                     raise InvalidArgument(message % (name,))
             yield
         else:
-            # Warn about function-scoped fixtures, excluding autouse fixtures because
-            # the advice is probably not actionable and the status quo seems OK...
-            # See https://github.com/HypothesisWorks/hypothesis/issues/377 for detail.
-            msg = (
-                "%s uses the %r fixture, which is reset between function calls but not "
-                "between test cases generated by `@given(...)`.  You can change it to "
-                "a module- or session-scoped fixture if it is safe to reuse; if not "
-                "we recommend using a context manager inside your test function.  See "
-                "https://docs.pytest.org/en/latest/fixture.html#sharing-test-data "
-                "for details on fixture scope."
-            )
-            argnames = None
-            for fx_defs in item._request._fixturemanager.getfixtureinfo(
-                node=item, func=item.function, cls=None
-            ).name2fixturedefs.values():
-                if argnames is None:
-                    argnames = frozenset(signature(item.function).parameters)
-                for fx in fx_defs:
-                    if fx.argname in argnames:
-                        active_fx = item._request._get_active_fixturedef(fx.argname)
-                        if active_fx.scope == "function":
-                            note_deprecation(
-                                msg % (item.nodeid, fx.argname), since="2020-02-29"
-                            )
+            # Retrieve the settings for this test from the test object, which
+            # is normally a Hypothesis wrapped_test wrapper. If this doesn't
+            # work, the test object is probably something weird
+            # (e.g a stateful test wrapper), so we skip the function-scoped
+            # fixture check.
+            settings = getattr(item.obj, "_hypothesis_internal_use_settings", None)
+
+            # Check for suspicious use of function-scoped fixtures, but only
+            # if the corresponding health check is not suppressed.
+            if (
+                settings is not None
+                and HealthCheck.function_scoped_fixture
+                not in settings.suppress_health_check
+            ):
+                # Warn about function-scoped fixtures, excluding autouse fixtures because
+                # the advice is probably not actionable and the status quo seems OK...
+                # See https://github.com/HypothesisWorks/hypothesis/issues/377 for detail.
+                msg = (
+                    "%s uses the %r fixture, which is reset between function calls but not "
+                    "between test cases generated by `@given(...)`.  You can change it to "
+                    "a module- or session-scoped fixture if it is safe to reuse; if not "
+                    "we recommend using a context manager inside your test function.  See "
+                    "https://docs.pytest.org/en/latest/fixture.html#sharing-test-data "
+                    "for details on fixture scope."
+                )
+                argnames = None
+                for fx_defs in item._request._fixturemanager.getfixtureinfo(
+                    node=item, func=item.function, cls=None
+                ).name2fixturedefs.values():
+                    if argnames is None:
+                        argnames = frozenset(signature(item.function).parameters)
+                    for fx in fx_defs:
+                        if fx.argname in argnames:
+                            active_fx = item._request._get_active_fixturedef(fx.argname)
+                            if active_fx.scope == "function":
+                                note_deprecation(
+                                    msg % (item.nodeid, fx.argname),
+                                    since="2020-02-29",
+                                    has_codemod=False,
+                                )
 
             if item.get_closest_marker("parametrize") is not None:
                 # Give every parametrized test invocation a unique database key
